import acedeploy.core.model_instance_objects as mio
import pytest


@pytest.mark.parametrize(
    "table1metadata, table2metadata, expected_result",
    [
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            True,
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            True,  # clustering key
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(t4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            True,  # clustering key
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "( T4) ",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            True,  # clustering key
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T3)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": "(T4)",
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # clustering key
        ),
        # (
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 1, "COMMENT": None, "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}], 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],},
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 1, "COMMENT": "", "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}]}, 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],,
        #     True # comment None vs emtpy string
        # ),
        # (
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 1, "COMMENT": "test comment", "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}], 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],},
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 1, "COMMENT": "", "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}], 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],},
        #     False # comment
        # ),
        # (
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 1, "COMMENT": "", "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}], 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],},
        #     {"DATABASE_NAME": "TWZ_META", "SCHEMA_NAME": "DATA", "OBJECT_NAME": "TABLE2", "TABLE_TYPE": "BASE TABLE", "CLUSTERING_KEY": None, "RETENTION_TIME": 90, "COMMENT": "", "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}], 'constraint_foreign_keys': [], 'constraint_primary_keys': [], 'constraint_unique_keys': [],},
        #     False # retention time
        # ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # column id 4 is missing in first table
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # ordinal position 3 and 4 switched
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":32,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # column T4 CHARACTER_OCTET_LENGTH differs
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 90,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # retention time differs
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 90,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # schema retention time differs
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "one",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "two",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # comment differs
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T4"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": []
            },
            False, # row access policy only on table1
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T4"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            False, # row access policy only on table2
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T4"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T4"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            True, # same row access policy on both
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T4"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "ROW_ACCESS_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "", "REF_ARG_COLUMN_NAMES": '["T5"]', "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            False, # different row access policies (differing REF_ARG_COLUMN_NAMES)
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T4", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": []
            },
            False, # masking policy only on table1
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T4", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            False, # masking policy only on table2
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T4", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T4", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            True, # same masking policy on both tables
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T4", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
                "policy_references": [{"POLICY_DB": "my_db", "POLICY_SCHEMA": "my_schema", "POLICY_NAME": "my_policy", "POLICY_KIND": "MASKING_POLICY", "REF_DATABASE_NAME": "my_db", "REF_SCHEMA_NAME": "DATA", "REF_ENTITY_NAME": "TABLE2", "REF_ENTITY_DOMAIN": "TABLE", "REF_COLUMN_NAME": "T5", "REF_ARG_COLUMN_NAMES": "", "TAG_DATABASE": "", "TAG_SCHEMA": "", "TAG_NAME": "", "POLICY_STATUS": "",}]
            },
            False, # masking policy has changed (REF_COLUMN_NAME)
        ),
    ],
)
def test_tables_equality(table1metadata, table2metadata, expected_result):
    table1 = mio.InstanceTable(table1metadata)
    table2 = mio.InstanceTable(table2metadata)
    result = table1 == table2
    assert result == expected_result


@pytest.mark.parametrize(
    "table1metadata, table2metadata, expected_result",
    [
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            True,  # ordinal position 3 and 4 switched, ignore_column_order=True for first table
        ),
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":32,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # column T4 CHARACTER_OCTET_LENGTH differs, ignore_column_order=True
        ),
    ],
)
def test_tables_equality_ignore_order1(table1metadata, table2metadata, expected_result):
    table1 = mio.InstanceTable(table1metadata, ignore_column_order=True)
    table2 = mio.InstanceTable(table2metadata)
    result = table1 == table2
    assert result == expected_result


@pytest.mark.parametrize(
    "table1metadata, table2metadata, expected_result",
    [
        (
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            {
                "DATABASE_NAME": "TWZ_META",
                "SCHEMA_NAME": "DATA",
                "OBJECT_NAME": "TABLE2",
                "TABLE_TYPE": "BASE TABLE",
                "CLUSTERING_KEY": None,
                "RETENTION_TIME": 1,
                "SCHEMA_RETENTION_TIME": 1,
                "COMMENT": "",
                "COLUMN_DETAILS": [{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLLATION_NAME":"en-ci","COLUMN_DEFAULT":"\'x\'","COLUMN_NAME":"T4","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":4,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T2","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":2,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":11,"CHARACTER_OCTET_LENGTH":44,"COLUMN_NAME":"T1","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"YES","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":1,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"},{"CHARACTER_MAXIMUM_LENGTH":12,"CHARACTER_OCTET_LENGTH":48,"COLLATION_NAME":"en-ci","COLUMN_NAME":"T3","DATABASE_NAME":"TWZ_META","DATA_TYPE":"TEXT","IS_IDENTITY":"NO","IS_NULLABLE":"NO","IS_SELF_REFERENCING":"NO","ORDINAL_POSITION":3,"TABLE_NAME":"TABLE2","TABLE_SCHEMA":"DATA"}],
                "constraint_foreign_keys": [],
                "constraint_primary_keys": [],
                "constraint_unique_keys": [],
            },
            False,  # ordinal position 3 and 4 switched, ignore_column_order=True for second table
        )
    ],
)
def test_tables_equality_ignore_order2(table1metadata, table2metadata, expected_result):
    table1 = mio.InstanceTable(table1metadata)
    table2 = mio.InstanceTable(table2metadata, ignore_column_order=True)
    result = table1 == table2
    assert result == expected_result
